# Openvpn helper things
from django.conf import settings

import os


def _readlog(statuslog):
    clients = {}
    with open(statuslog) as log:
        current_stat = None

        for ln in log:
            if ln.startswith('Common Name'):
                current_stat = 'clients'

            elif ln.startswith('Virtual Address'):
                current_stat = 'routes'

            elif ln.startswith('GLOBAL') or ln.startswith('ROUTING'):
                current_stat = None

            elif current_stat == 'clients':
                name, remote_ip, rx, tx, start = ln.strip().split(',')

                clients[name] = {
                    'ip': remote_ip.split(':')[0],
                    'rx_bytes': int(rx),
                    'tx_bytes': int(tx),
                    'connected': start
                }

            elif current_stat == 'routes':
                virtual, name, remote, start = ln.strip().split(',')
                clients[name]['virtual'] = virtual
    return clients

def getStatus():

    statuslog = os.path.join(settings.BASE_PATH, 'openvpn-status.log')

    if not os.path.exists(statuslog):
        return {}

    try:
        clients = _readlog(statuslog)
    except PermissionError as err:
        # Try to fix it
        from gamoto import users
        users.sudo(
            'chown', 
            '%s:%s' % (settings.GAMOTO_USER, settings.GAMOTO_GROUP),
            statuslog
        )
        clients = _readlog(statuslog)

    return clients


def getClient(name):
    return getStatus().get(name, None)



from django.core.management.base import BaseCommand, CommandError
from django.conf import settings
from gamoto import ca, users

import os


class Command(BaseCommand):
    help = 'Configure the CA'

    def handle(self, *args, **options):
        def _status(var):
            if var:
                self.stdout.write(self.style.SUCCESS('OK'))
            else:
                self.stdout.write(self.style.SUCCESS('EXISTS'))

        uid = users.getSystemUID()
        my_uid = os.getuid()
        if (my_uid == 0):
            os.setuid(uid)

        elif (my_uid != uid):
            self.stdout.write(self.style.ERROR(
                "Please run this command as root, or " + settings.GAMOTO_USER))
            return

        myca = ca.CertificateAuthority(
            settings.CA_SETUP['ou'],
            settings.CA_SETUP['org'],
            settings.CA_SETUP['email'],
            settings.CA_SETUP['country'],
            settings.CA_SETUP['state'],
            settings.CA_SETUP['city'],
            key_path=settings.CA_PATH
        )

        self.stdout.write("Creating Diffie-Hellman parameters... ", ending="")
        self.stdout.flush()

        _status(myca.createDH())

        self.stdout.write("Building certificate authority... ", ending="")
        self.stdout.flush()
        _status(myca.createCA('openvpn'))

        self.stdout.write("Creating server certificate request... ", ending="")
        self.stdout.flush()
        _status(myca.createCSR('openvpn'))

        self.stdout.write("Signing server certificate... ", ending="")
        self.stdout.flush()
        _status(myca.signCSR('openvpn', server=True))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib.auth.decorators import login_required
from django.http import HttpResponse
from django.shortcuts import render, redirect
from django.contrib.auth import views as auth_views

from gamoto import users, openvpn


def returnFile(name, content_type, data):
    response = HttpResponse(data, content_type=content_type)
    response['Content-Disposition'] = 'attachment; filename="%s"' % name
    return response


@login_required
def index(request):
    user_name = request.user.username

    passwd = users.getUser(user_name)

    vpn = openvpn.getClient(user_name)

    return render(request, "index.html", {
        'passwd': passwd,
        'vpn': vpn,
        'name': request.user.get_full_name()
    })


@login_required
def vpn_zip(request):
    user_name = request.user.username
    passwd = users.getUser(user_name)

    return returnFile(user_name + ".zip", "application/zip",
                      users.getVPNZIP(user_name))


@login_required
def vpn_tblk(request):
    user_name = request.user.username
    passwd = users.getUser(user_name)

    return returnFile(user_name + ".tblk", "application/zip", "test")


@login_required
def reset_2fa(request):
    user_name = request.user.username
    codes, authurl = users.configureTOTP(user_name)

    return render(request, "enroll.html", {
        'name': request.user.get_full_name(),
        'authurl': authurl,
        'codes': codes,
        'reset': True
    })


@login_required
def enroll_user(request):
    user_name = request.user.username
    passwd = users.getUser(user_name)

    if passwd:
        return redirect('index')

    users.createUser(user_name)
    users.createVPN(user_name)
    codes, authurl = users.configureTOTP(user_name)

    return render(request, "enroll.html", {
        'name': request.user.get_full_name(),
        'authurl': authurl,
        'codes': codes
    })
